
#include <stdio.h>

#include "vocabs.hpp"
#include "server.hpp"

using namespace yarp;
using namespace yarp::os;
using namespace std;


CollatzServerProcessor::CollatzServerProcessor(const unsigned int dumpPeriod) :
                        RateThread(dumpPeriod)
{
    curNum=0;
    mutex=new Semaphore(1);
}


CollatzServerProcessor::~CollatzServerProcessor()
{
    delete mutex;
}


void CollatzServerProcessor::tickItem(const unsigned int num)
{
    list<unsigned int>::iterator itr;
    bool warn=true;

    // scan the table and remove the item verified by the client
    mutex->wait();
    for (itr=table.begin(); itr!=table.end(); itr++)
        if (*itr==num)
        {
            table.erase(itr);
            warn=false;
            break;
        }
    mutex->post();

    if (warn)
        fprintf(stdout,"Warning: unknown item %d\n",num);
}


void CollatzServerProcessor::generateItem(unsigned int &num, unsigned int &thres)
{
    mutex->wait();
    table.push_back(++curNum);

    num=curNum;

    // we know that front-1 has been already verified
    thres=table.front()-1;
    mutex->post();
}


void CollatzServerProcessor::dumpTable()
{
    list<unsigned int>::iterator itr;

    fprintf(stdout,"Items under verification:\n");

    // dump the table content for monitoring:
    // if any item remains in the table for long
    // it means that it might be a counterexample for
    // the conjecture (but that won't be the case :)
    mutex->wait();
    if (table.empty())
        fprintf(stdout,"none");
    else for (itr=table.begin(); itr!=table.end(); itr++)
        fprintf(stdout,"%d ",*itr);
    mutex->post();

    fprintf(stdout,"\n\n");
}


bool CollatzServerProcessor::read(ConnectionReader& connection)
{
    Bottle in;

    // get the request from the client
    in.read(connection);

    // process the request
    if (in.get(0).asVocab()==COLLATZ_VOCAB_REQ_ITEM)
    {
        unsigned int num, thres;

        num=in.get(1).asInt();

        // if the request contains a valid item,
        // then tick the item out of the table
        if (num!=COLLATZ_EMPTY_FIELD)
            tickItem(num);

        // generate a new item
        generateItem(num,thres);
    
        Bottle out;
        out.addVocab(COLLATZ_VOCAB_ITEM);
        out.addInt((int)num);
        out.addInt((int)thres);
    
        // send the item to the client
        if (ConnectionWriter *returnToSender=connection.getWriter())
            out.write(*returnToSender);
    }

    return true;
}


void CollatzServerProcessor::run()
{
    dumpTable();
}


